﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PSOCore
{
    public class MaxSwarm:Swarm
    {
        public double bestGlobalFitness = double.MinValue;
        MaxParticle[] maxswarm;
        public MaxSwarm(int dim, int numberParticles)
            : base(dim, numberParticles)
        { 
            maxswarm = new MaxParticle[numberParticles];          
        }
        public override void inicializace(Int32 fce)
        {
            fn = new FunctionBase(fce);
            for (int i = 0; i < maxswarm.Length; ++i) 
            {
                double[] randomPosition = new double[pocetDimenzi];
                for (int j = 0; j < randomPosition.Length; ++j)
                {
                    double lo = minX;
                    double hi = maxX;
                    randomPosition[j] = (hi - lo) * _rnd.NextDouble() + lo; 
                }

                double fitness = fn.Function(randomPosition);
                double[] randomVelocity = new double[pocetDimenzi];

                for (int j = 0; j < randomVelocity.Length; ++j)
                {
                    double lo = -1.0 * Math.Abs(maxX - minX);
                    double hi = Math.Abs(maxX - minX);
                    randomVelocity[j] = (hi - lo) * _rnd.NextDouble() + lo;
                }
                maxswarm[i] = new MaxParticle(randomPosition, fitness, randomVelocity, randomPosition, fitness);

 
                if (maxswarm[i].fitness > bestGlobalFitness)
                {
                    bestGlobalFitness = maxswarm[i].fitness;
                    maxswarm[i].pozice.CopyTo(bestGlobalPozice, 0);
                }
            } 
            this.minR = -1.0 * maxX;
            this.maxR = maxX;
        }
        public override void psoLoop()
        {
            FormBar formular_nacitani = new FormBar();
            formular_nacitani.Show();
            Swarm.reseni += Environment.NewLine;
            Swarm.reseni += "Průběh výpočtu";
            Swarm.reseni += Environment.NewLine;
            Swarm.reseni += "Rychlost a pozice náhodné částice";
            Swarm.reseni += Environment.NewLine;
            while (iteraci < pocetIteraci)
            {
                ++iteraci;
                double[] newVelocity = new double[pocetDimenzi];
                double[] newPosition = new double[pocetDimenzi];

                for (int i = 0; i < maxswarm.Length; ++i) 
                {
                    maxswarm[i].MaximalizeUpdateRychlost(ref maxswarm[i], this, ref newVelocity,
                    ref newPosition, ref this.bestGlobalPozice, ref this.bestGlobalFitness);
                }
                if (iteraci % 5 == 0)
                {
                    Swarm.reseni += Environment.NewLine;
                    Swarm.reseni += "Iterace " + iteraci;
                    Swarm.reseni += Environment.NewLine;
                    Swarm.reseni += maxswarm[0].ToString();
                }
                pbar = this.SetPBValue();
                formular_nacitani.nastav(pbar);
                formular_nacitani.Refresh();
                if (formular_nacitani.vratpBar() == 100)
                {
                    formular_nacitani.Dispose();
                    formular_nacitani.Close();
                    break;
                }

            }
            if (Swarm.sekvence != "")
            {
                Swarm.reseni += Environment.NewLine;
                Swarm.reseni += "Nejdelší nalezená podsekvence " + Swarm.sekvence;
                Swarm.reseni += Environment.NewLine;
                for (int k = 0; k<Swarm.vypis.Length; k++) {
                   // Console.WriteLine(Swarm.vypis[k]);
                    if (Swarm.vypis[k].Contains(Swarm.sekvence))
                    {
                        Swarm.vypis[k] = Swarm.vypis[k].Replace(" ", "");
                        Swarm.reseni += "Obsažena v sekvenci na řádku " + (k + 1);
                        Swarm.spolecneRadky += (k + 1) + ", ";
                        Swarm.reseni += Environment.NewLine;                       
                    }
                    Swarm.vypis[k] = "";
                }
                Swarm.reseni += Environment.NewLine;               
            }
            Swarm.reseni += "Nejlepší nalezená pozice ";
            for (int l = 0; l < this.bestGlobalPozice.Length; l++) { 
                Swarm.reseni += this.bestGlobalPozice[l]+" | ";
            }
            Swarm.reseni += Environment.NewLine;
            Swarm.reseni += "Nejvyšší dosažené fitness ze všech částic " + this.bestGlobalFitness;
            Swarm.reseni += Environment.NewLine;
            Swarm.bestValue = this.bestGlobalFitness;
        }
    }
}
