﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PSOCore
{
    public class Particle
    {
        #region StatickeVlastnosti
        protected static Random _rnd = new Random();
        #endregion

        #region Vlastnosti
        public double[] pozice; 
        public double fitness;
        public double[] rychlost;

        public double[] bestPozice; 
        public double bestFitness;

        #endregion

        #region Metody
        public Particle(double[] position, double fitness, double[] velocity, double[] bestPosition, double bestFitness)
        {
            this.pozice = new double[position.Length];
            position.CopyTo(this.pozice, 0);
            this.fitness = fitness;
            this.rychlost = new double[velocity.Length];
            velocity.CopyTo(this.rychlost, 0);
            this.bestPozice = new double[bestPosition.Length];
            bestPosition.CopyTo(this.bestPozice, 0);
            this.bestFitness = bestFitness;
        }

        public virtual void UpdateRychlost(ref Particle actualParticle,Swarm swarm, ref double[] novaRychlost, 
            ref double[] newPosition, ref double[] bestGlobalPosition, ref double bestGlobalFitness)
        {
            double r1, r2,newFitness;           
            for (int j = 0; j < actualParticle.rychlost.Length; ++j) 
            {
                r1 = _rnd.NextDouble();
                r2 = _rnd.NextDouble();

                novaRychlost[j] = (Swarm.vaha * actualParticle.rychlost[j]) +
                  (Swarm.constP * r1 * (actualParticle.bestPozice[j] - actualParticle.pozice[j])) +
                  (Swarm.constG * r2 * (bestGlobalPosition[j] - actualParticle.pozice[j]));

                if (novaRychlost[j] < swarm.minR)
                    novaRychlost[j] = swarm.minR;
                else if (novaRychlost[j] > swarm.maxR)
                    novaRychlost[j] = swarm.maxR;
            }

            novaRychlost.CopyTo(actualParticle.rychlost, 0);

            for (int j = 0; j < actualParticle.pozice.Length; ++j)
            {
                newPosition[j] = actualParticle.pozice[j] + novaRychlost[j];
                if (newPosition[j] < Swarm.minX)
                    newPosition[j] = Swarm.minX;
                else if (newPosition[j] > Swarm.maxX)
                    newPosition[j] = Swarm.maxX;
            }

            newPosition.CopyTo(actualParticle.pozice, 0);
            newFitness = swarm.fn.Function(newPosition);
            actualParticle.fitness = newFitness;

            if (newFitness > actualParticle.bestFitness)
            {
                newPosition.CopyTo(actualParticle.bestPozice, 0);
                actualParticle.bestFitness = newFitness;
            }

            if (newFitness > bestGlobalFitness)
            {
                newPosition.CopyTo(bestGlobalPosition, 0);
                bestGlobalFitness = newFitness;
            }
        }

        public override string ToString()
        {
            string s = "";
            s += "==========================\n";
            s += "Pozice: ";
            for (int i = 0; i < this.pozice.Length; ++i)
                s += this.pozice[i].ToString("F2") + " ";
            s += "\n";
            s += "Fitness při aktuální pozici = " + this.fitness.ToString("F4") + "\n";
            s += "Rychlost: ";
            for (int i = 0; i < this.rychlost.Length; ++i)
                s += this.rychlost[i].ToString("F2") + " ";
            s += "\n";
            s += "Pozice při nejlepším výsledku: ";
            for (int i = 0; i < this.bestPozice.Length; ++i)
            {
                s += this.bestPozice[i].ToString("F2") + " ";                     
            }
            s += "\n";
            s += "Nejlepší dosažený výsledek této částice= " + this.bestFitness.ToString("F4") + "\n";
            s += "==========================\n";
            return s;
        }

        #endregion
    }
}
